--------------------------------------
-- Namespaces
--------------------------------------
local _, core = ...;
core.Config = {}; -- adds Config table to addon namespace

local Config = core.Config;
local UIConfig;

local UIViewer;
UIViewer = CreateFrame("Frame", "UIViewer", UIParent, "UIPanelDialogTemplate");
UIViewer:SetMovable(true);
UIViewer:EnableMouse(true);
UIViewer:RegisterForDrag("LeftButton");
UIViewer:SetScript("OnDragStart", UIViewer.StartMoving);
UIViewer:SetScript("OnDragStop", UIViewer.StopMovingOrSizing);
UIViewer:SetSize(150, 400);
UIViewer:SetPoint("TOPLEFT"); -- Doesn't need to be ("CENTER", UIParent, "CENTER")
UIViewer:SetShown(false);

local timer = 0.25;
UIViewer:SetScript("OnUpdate", function(self, elapsed)
	timer = timer - elapsed
	if (timer > 0) then return end
	getDebuffs();
	timer = timer + 0.25
end
);
--------------------------------------
-- Defaults (usually a database!)
--------------------------------------
local defaults = {
	theme = {
		r = 0, 
		g = 0.8, -- 204/255
		b = 1,
		hex = "00ccff"
	}
}

--------------------------------------
-- Config functions
--------------------------------------
function Config:Toggle()
	local menu = UIConfig or Config:CreateMenu();
	menu:SetShown(not menu:IsShown());
end

function UIViewerToggle()
	UIViewer:SetShown(not UIViewer:IsShown());
end

function Config:GetThemeColor()
	local c = defaults.theme;
	return c.r, c.g, c.b, c.hex;
end

function Config:CreateButton(point, relativeFrame, relativePoint, yOffset, text)
	local btn = CreateFrame("Button", nil, UIConfig.ScrollFrame, "GameMenuButtonTemplate");
	btn:SetPoint(point, relativeFrame, relativePoint, 0, yOffset);
	btn:SetSize(140, 40);
	btn:SetText(text);
	btn:SetNormalFontObject("GameFontNormalLarge");
	btn:SetHighlightFontObject("GameFontHighlightLarge");
	return btn;
end

local function ScrollFrame_OnMouseWheel(self, delta)
	local newValue = self:GetVerticalScroll() - (delta * 20);
	
	if (newValue < 0) then
		newValue = 0;
	elseif (newValue > self:GetVerticalScrollRange()) then
		newValue = self:GetVerticalScrollRange();
	end
	
	self:SetVerticalScroll(newValue);
end

function Config:CreateMenu()
	UIConfig = CreateFrame("Frame", "AuraTrackerConfig", UIParent, "UIPanelDialogTemplate");
    UIConfig:SetSize(250, 400);
    UIConfig:SetMovable(true);
    UIConfig:EnableMouse(true);
    UIConfig:RegisterForDrag("LeftButton");
    UIConfig:SetScript("OnDragStart", UIConfig.StartMoving);
    UIConfig:SetScript("OnDragStop", UIConfig.StopMovingOrSizing);
	UIConfig:SetPoint("CENTER"); -- Doesn't need to be ("CENTER", UIParent, "CENTER")

	UIConfig.Title:ClearAllPoints();
	UIConfig.Title:SetFontObject("GameFontHighlight");
	UIConfig.Title:SetPoint("LEFT", AuraTrackerConfigTitleBG, "LEFT", 6, 1);
	UIConfig.Title:SetText("Loatheb Debuffs");

	local OpenButton = CreateFrame("Button", "OpenButton", UIConfig, "UIPanelButtonTemplate");
	OpenButton:SetWidth(90);
	OpenButton:SetHeight(20);
	OpenButton:SetPoint("TOPRIGHT", UIConfig,"TOPRIGHT", -27, -5);
	OpenButton:SetText("Open Debuffs");
	OpenButton:SetScript("OnClick", function(self)
		updateUIViewer();
		UIViewer:SetShown(true)
	end);

	UIConfig.ScrollFrame = CreateFrame("ScrollFrame", nil, UIConfig, "UIPanelScrollFrameTemplate");
	UIConfig.ScrollFrame:SetPoint("TOPLEFT", AuraTrackerConfigDialogBG, "TOPLEFT", 4, -8);
	UIConfig.ScrollFrame:SetPoint("BOTTOMRIGHT", AuraTrackerConfigDialogBG, "BOTTOMRIGHT", -3, 4);
	UIConfig.ScrollFrame:SetClipsChildren(true);
	UIConfig.ScrollFrame:SetScript("OnMouseWheel", ScrollFrame_OnMouseWheel);
	
	UIConfig.ScrollFrame.ScrollBar:ClearAllPoints();
	UIConfig.ScrollFrame.ScrollBar:SetPoint("TOPLEFT", UIConfig.ScrollFrame, "TOPRIGHT", -12, -18);
	UIConfig.ScrollFrame.ScrollBar:SetPoint("BOTTOMRIGHT", UIConfig.ScrollFrame, "BOTTOMRIGHT", -7, 18);

	local child = CreateFrame("Frame", "ChildFrame", UIConfig.ScrollFrame);
	child:SetSize(208, 500);
	UIConfig.ScrollFrame:SetScrollChild(child);	


	local ShowLegend = UIConfig:CreateFontString("ShowLegend", UIConfig, "GameFontNormal")
	ShowLegend:SetText("Show")
	ShowLegend:SetPoint("TOPLEFT", UIConfig, "TOPLEFT", 12, -30);
	local NameLegend = UIConfig:CreateFontString("NameLegend", UIConfig, "GameFontNormal")
	NameLegend:SetText("Name")
	NameLegend:SetPoint("TOPLEFT", UIConfig, "TOPLEFT", 67, -30);
	local OrderLegend = UIConfig:CreateFontString("OrderLegend", UIConfig, "GameFontNormal")
	OrderLegend:SetText("Order")
	OrderLegend:SetPoint("TOPLEFT", UIConfig, "TOPLEFT", 127, -30);

--[[	
	----------------------------------
	-- Buttons
	----------------------------------
	-- Save Button:
	UIConfig.saveBtn = self:CreateButton("CENTER", child, "TOP", -70, "Save");

	-- Reset Button:	
	UIConfig.resetBtn = self:CreateButton("TOP", UIConfig.saveBtn, "BOTTOM", -10, "Reset");

	-- Load Button:	
	UIConfig.loadBtn = self:CreateButton("TOP", UIConfig.resetBtn, "BOTTOM", -10, "Load");

	----------------------------------
	-- Sliders
	----------------------------------
	-- Slider 1:
	UIConfig.slider1 = CreateFrame("SLIDER", nil, UIConfig.ScrollFrame, "OptionsSliderTemplate");
	UIConfig.slider1:SetPoint("TOP", UIConfig.loadBtn, "BOTTOM", 0, -20);
	UIConfig.slider1:SetMinMaxValues(1, 100);
	UIConfig.slider1:SetValue(50);
	UIConfig.slider1:SetValueStep(30);
	UIConfig.slider1:SetObeyStepOnDrag(true);

	-- Slider 2:
	UIConfig.slider2 = CreateFrame("SLIDER", nil, UIConfig.ScrollFrame, "OptionsSliderTemplate");
	UIConfig.slider2:SetPoint("TOP", UIConfig.slider1, "BOTTOM", 0, -20);
	UIConfig.slider2:SetMinMaxValues(1, 100);
	UIConfig.slider2:SetValue(40);
	UIConfig.slider2:SetValueStep(30);
	UIConfig.slider2:SetObeyStepOnDrag(true);
]]--
	----------------------------------
	-- Check Buttons
	----------------------------------
    -- Check Button 1:

	local ypos = -30

	for i = 1, 40 do
		local checkBtn = CreateFrame("CheckButton", "CheckButton" .. i, UIConfig.ScrollFrame, "UICheckButtonTemplate");
		

		local addText = UIViewer:CreateFontString("text" .. i, UIViewer, "GameFontNormal")
		addText:SetText(i)
        checkBtn:SetPoint("TOPLEFT", child, "TOPLEFT", 7, ypos);
		checkBtn.text:SetText(i);

		local f = CreateFrame("Frame", "debuff" .. i, UIViewer)

		f:SetHeight(16)
		f:SetWidth(16)
		local t = f:CreateTexture("texture" .. i, UIViewer)
		t:SetTexture("Interface\\AddOns\\LoathebDebuffs\\Images\\corMind.blp")		
		t:SetAllPoints(f)
		f.texture = t
		ypos = ypos-30;
		
		local eb = CreateFrame("EditBox", "EditBox" .. i, UIConfig.ScrollFrame, "InputBoxTemplate")
		eb:SetHeight(16)
		eb:SetWidth(16)
		eb:SetPoint("RIGHT", checkBtn, "RIGHT", 100, 0);
		eb:SetAutoFocus();

		checkBtn:SetScript("OnClick", 
		function()
			if checkBtn:GetChecked() then
				eb:Show();
				saveTable();
			else
				eb:Hide();
			end
		end);
		
		eb:Hide();
		checkBtn:Hide();
	end
	
	--updateUIConfig();

    UIConfig:SetShown(false);
    
    return UIConfig;
end

_G["SLASH_ldo1"] = "/ldo";
SlashCmdList["ldo"] = function(cmd)	
	Config:Toggle()
	updateUIConfig();
	_G["ChildFrame"]:SetSize(150, (GetNumGroupMembers()*30)+10);
end

_G["SLASH_ld1"] = "/ld";
SlashCmdList["ld"] = function(cmd)

	updateUIViewer()

    if UIViewer:IsShown() == nil then
        UIViewer:SetShown(true)
    else
        UIViewerToggle();
    end	
end

function getDebuffs()
	if UIViewer:IsShown() then
		for k1 = 1, GetNumGroupMembers() do
			local found = 0;
			local indexer = 1;
			local name, realm = UnitName("raid" .. k1)
			for k2 = 1, 40 do
				local Sname, Srank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId 
					= UnitDebuff("raid" .. k1, k2)
				if Sname == "Corrupted Mind" then
					found = found + 1;
				end
				if name == _G["text" .. k2]:GetText() then
					indexer = k2	
				end
			end
			if (found == 2) then
				_G["debuff" .. indexer]:Show();
			else
				_G["debuff" .. indexer]:Hide();
			end
		end
	end
end

function tablelength(T)
	local count = 0
	for _ in pairs(T) do count = count + 1 end
	return count
end

function saveTable()
	for i = 1, 40 do
		for index,values in pairs(RaidTable) do
			if (index == _G[tostring("CheckButton" .. i)].text:GetText(i)) and _G[tostring("CheckButton" .. i)]:GetChecked() then
				values.isChecked = true
				values.spot = _G[tostring("EditBox" .. i)]:GetNumber() 
			end
			if (index == _G[tostring("CheckButton" .. i)].text:GetText(i)) and not _G[tostring("CheckButton" .. i)]:GetChecked() then
				values.isChecked = false
			end
		end
	end
end

function updateUIViewer()
	local yPosition = -30

	saveTable();
	
	for i = 1, 40 do
		_G["text" .. i]:Hide();
		_G["debuff" .. i]:Hide();
	end

	local RaidTableSorted, RaidTableTemp;
	RaidTableTemp = {};
	RaidTableSorted = {};
	for i, v in pairs(RaidTable) do
		if (v.spot == 0 or v.spot == nil) and v.isChecked then
			RaidTableTemp[i] = 0
		else
			RaidTableTemp[i] = v.spot
		end
	end

	for k,v in spairs(RaidTableTemp, function(t,a,b) return t[b] > t[a] end) do
		RaidTableSorted[k] = v
	end

	for ind, val in spairs(RaidTableSorted, function(t,a,b) return t[b] > t[a] end) do		
		for j = 1, GetNumGroupMembers() do
			local name, rank, subgroup, level, class, fileName, 
			zone, online, isDead, role, isML = GetRaidRosterInfo(j);

			if name == ind then
				yPosition = yPosition -30
				_G["text" .. j]:SetText(name)
				_G["text" .. j]:SetPoint("TOPLEFT", 30, yPosition)
				_G["text" .. j]:Show();
				_G["debuff" .. j]:SetPoint("TOPLEFT", UIViewer, 120, yPosition+4)
				_G["debuff" .. j]:Hide();
				UIViewer:SetSize(150, 60-yPosition);
			end
		end
    end
end

function updateUIConfig()
	for i = 1, GetNumGroupMembers() do
		local name, rank, subgroup, level, class, fileName, 
		zone, online, isDead, role, isML = GetRaidRosterInfo(i);

		if RaidTable == nil then
			RaidTable = {}
			RaidTable[name] = {isChecked = false, spot = nil}
		else
			local foundIndex = false
			for index,values in pairs(RaidTable) do
				if index == name then
					foundIndex = true
				end
			end
			if not foundIndex then
				RaidTable[name] = {isChecked = false, spot = nil}
			end
		end
		_G[tostring("CheckButton" .. i)].text:SetText(name);
		_G[tostring("CheckButton" .. i)]:SetChecked(RaidTable[name].isChecked);

		if _G[tostring("CheckButton" .. i)]:GetChecked() then
			_G[tostring("EditBox" .. i)]:Show();
		else
			_G[tostring("EditBox" .. i)]:Hide();
		end

		if RaidTable[name].spot == 0 or RaidTable[name].spot == nil then
			_G[tostring("EditBox" .. i)]:SetText("");
		else
			_G[tostring("EditBox" .. i)]:SetText(tostring(RaidTable[name].spot));
		end

		_G[tostring("CheckButton" .. i)]:Show();
		--_G[tostring("EditBox" .. i)]:Show();
	end
end

function spairs(t, order)
    -- collect the keys
    local keys = {}
    for k in pairs(t) do keys[#keys+1] = k end

    -- if order function given, sort by it by passing the table and keys a, b,
    -- otherwise just sort the keys 
    if order then
        table.sort(keys, function(a,b) return order(t, a, b) end)
    else
        table.sort(keys)
    end

    -- return the iterator function
    local i = 0
    return function()
        i = i + 1
        if keys[i] then
            return keys[i], t[keys[i]]
        end
    end
end

Config:CreateMenu();
